@@ -9,10 +9,12 @@
 #define DBG_FILE	0x0001
 #define DBG_LOAD	0x0002
 #define DBG_LDR 	0x0004
+#define DBG_LDR2	0x0008
 
 #if __LOADER_COMMON_H
 #undef D
-#define D(fmt...) DC(DBG_LDR, fmt)
+#define D(fmt...)  DC(DBG_LDR, fmt)
+#define DL(fmt...) DC(DBG_LDR2, fmt)
 #endif
 
 __EXPORT__ extern unsigned int __imlib_debug;
@@ -26,6 +28,9 @@ unsigned int        __imlib_time_us(void);
 
 #define D(fmt...)
 #define DC(fmt...)
+#if __LOADER_COMMON_H
+#define DL(fmt...)
+#endif
 
 #endif /* IMLIB2_DEBUG */
 
